package com.gree.mall.manager.logic.material.stock;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.manager.bean.admin.AdminUserCom;
import com.gree.mall.manager.bean.listvo.material.*;
import com.gree.mall.manager.commonmapper.CommonMapper;
import com.gree.mall.manager.enums.material.DirectFlagEnum;
import com.gree.mall.manager.enums.material.PartsAttrEnum;
import com.gree.mall.manager.enums.material.WebsitGoodsTypeEnum;
import com.gree.mall.manager.exception.RemoteServiceException;
import com.gree.mall.manager.logic.common.CommonLogic;
import com.gree.mall.manager.plus.entity.*;
import com.gree.mall.manager.plus.service.*;
import com.gree.mall.manager.utils.CommonUtils;
import com.gree.mall.manager.zfire.bean.ZfireParamBean;
import com.gree.mall.manager.zfire.util.FieldUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Slf4j
@RequiredArgsConstructor
@Service
public class WorkerStockLogic {

    private final CommonLogic commonLogic;
    private final CommonMapper commonMapper;
    private final WebsitGoodsService websitGoodsService;
    private final WorkerStockService workerStockService;
    private final WorkerStockAccService workerStockAccService;
    private final AdminWebsitService adminWebsitService;
    private final UserService userService;
    /**
     * 师傅辅材库存列表
     * @param zfireParamBean
     * @return
     */
    public IPage<WorkerStockVO> workerStockList(ZfireParamBean zfireParamBean){
        zfireParamBean.setCompanyWechatId(commonLogic.getAdminUser().getCompanyWechatId());
        FieldUtils.supplyParam(zfireParamBean,WorkerStockVO.class);

        IPage<WorkerStockVO> page = commonMapper.workerStockList(new Page(zfireParamBean.getPageNum(), zfireParamBean.getPageSize()), zfireParamBean);
        return page;
    }

    /**
     * 师傅辅材库存明细
     * @param zfireParamBean
     * @return
     */
    public IPage<WorkerStockAccVO> workerStockAccList(ZfireParamBean zfireParamBean){
        zfireParamBean.setCompanyWechatId(commonLogic.getAdminUser().getCompanyWechatId());
        FieldUtils.supplyParam(zfireParamBean,WorkerStockAccVO.class);
        IPage<WorkerStockAccVO> page = commonMapper.workerStockAccList(new Page(zfireParamBean.getPageNum(), zfireParamBean.getPageSize()), zfireParamBean);
        return page;
    }


    /**
     * 师傅配件库存列表
     * @param zfireParamBean
     * @return
     */
    public IPage<WorkerStockPVO> workerStockPList(ZfireParamBean zfireParamBean){
        zfireParamBean.setCompanyWechatId(commonLogic.getAdminUser().getCompanyWechatId());
        FieldUtils.supplyParam(zfireParamBean,WorkerStockPVO.class);
        IPage<WorkerStockPVO> page = commonMapper.workerStockPList(new Page(zfireParamBean.getPageNum(), zfireParamBean.getPageSize()), zfireParamBean);
        return page;
    }

    /**
     * 师傅配件库存明细
     * @param zfireParamBean
     * @return
     */
    public IPage<WorkerStockAccPVO> workerStockAccPList(ZfireParamBean zfireParamBean){
        zfireParamBean.setCompanyWechatId(commonLogic.getAdminUser().getCompanyWechatId());
        FieldUtils.supplyParam(zfireParamBean,WorkerStockAccPVO.class);
        IPage<WorkerStockAccPVO> page = commonMapper.workerStockAccPList(new Page(zfireParamBean.getPageNum(), zfireParamBean.getPageSize()), zfireParamBean);
        return page;
    }


    /**
     * 导入师傅库存(各个商品只能导第一次)
     * @param datas
     */
    public void importData(List<Object> datas, WebsitGoodsTypeEnum websitGoodsTypeEnum){
        AdminUserCom adminUser = commonLogic.getAdminUser();
        List<WorkerStock> workerStocks = new ArrayList<>();
        List<WorkerStockAcc> workerStockAccs = new ArrayList<>();
        for(int i= 0 ;i<datas.size() ; i++){
            List<String> row = (List<String>)datas.get(i);
            CommonUtils.initList(row,10);
            String goodsId = row.get(0);
            String stock = row.get(1);
            String wayQty = row.get(2);
            String oldQty = row.get(3);
            //String websitName = row.get(4);
            String workerMobile = row.get(4);
            if(StringUtils.isAnyBlank(goodsId,stock,wayQty,oldQty,workerMobile)){
                throw new RemoteServiceException("第"+(i+1)+"行请补充完整内容");
            }
            WebsitGoods websitGoods = websitGoodsService.lambdaQuery()
                    .eq(WebsitGoods::getCompanyWechatId,adminUser.getCompanyWechatId())
                    .eq(WebsitGoods::getGoodsType,websitGoodsTypeEnum.getKey())
                    .eq(WebsitGoods::getGoodsId,goodsId).one();
            if(websitGoods == null){
                throw new RemoteServiceException("商品编号【"+goodsId+"】不存在");
            }
//            AdminWebsit adminWebsit = adminWebsitService.lambdaQuery()
//                    .eq(AdminWebsit::getName, websitName)
//                    .eq(StringUtils.isNotBlank(adminUser.getCompanyWechatId()),AdminWebsit::getCompanyWechatId, adminUser.getCompanyWechatId()).one();
//            if(adminWebsit == null){
//                throw new RemoteServiceException("网点名字【"+websitName+"】不存在");
//            }
            //判断师傅信息
            User user = userService.lambdaQuery()
                    .eq(User::getMobile, workerMobile)
                    .eq(User::getCompanyWechatId, adminUser.getCompanyWechatId())
                    .last("limit 1").one();
            if(user == null){
                throw new RemoteServiceException("师傅手机号【"+workerMobile+"】不存在");
            }
            Integer count = workerStockService.lambdaQuery()
                    .eq(WorkerStock::getCompanyWechatId, adminUser.getCompanyWechatId())
                    .eq(WorkerStock::getWorkerId,user.getUserId())
                    .eq(WorkerStock::getGoodsId, goodsId).count();
            if(count > 0){
                throw new RemoteServiceException("商品编号【"+goodsId+"】的库存记录已存在于系统，导入失败");
            }

            WorkerStock bean = new WorkerStock();
            bean.setCompanyWechatId(adminUser.getCompanyWechatId());
            bean.setCompanyWechatName(adminUser.getCompanyName());
//            bean.setWebsitId(adminWebsit.getWebsitId());
//            bean.setWebsitName(adminWebsit.getName());
            bean.setGoodsType(websitGoodsTypeEnum.getKey());
            bean.setGoodsId(goodsId);
            bean.setGoodsName(websitGoods.getGoodsName());
            bean.setGoodsType(websitGoods.getGoodsType());
            bean.setQty(new BigDecimal(stock));
            bean.setWayQty(new BigDecimal(wayQty));
            bean.setOldQty(new BigDecimal(oldQty));
            bean.setWorkerId(user.getUserId());
            bean.setIdentity(user.getIdCard());
            bean.setCreateBy(adminUser.getNickName());
            bean.setUpdateBy(adminUser.getNickName());
            workerStocks.add(bean);

            //网点库存三级帐
            WorkerStockAcc workerStockAcc = new WorkerStockAcc();
            workerStockAcc.setCompanyWechatId(bean.getCompanyWechatId());
            workerStockAcc.setCompanyWechatName(bean.getCompanyWechatName());
//            workerStockAcc.setWebsitId(bean.getWebsitId());
//            workerStockAcc.setWebsitName(bean.getWebsitName());
            workerStockAcc.setGoodsType(bean.getGoodsType());
            workerStockAcc.setWorkerId(bean.getWorkerId());
//            workerStockAcc.setWorkerWebsitName(bean.getWebsitName());
//            workerStockAcc.setWorkerWebsitId(bean.getWebsitId());
            //workerStockAcc.setPartsAttr(PartsAttrEnum.NEW.getKey());
            workerStockAcc.setGoodsId(bean.getGoodsId());
            workerStockAcc.setGoodsName(bean.getGoodsName());
            workerStockAcc.setSdate(new Date());
            workerStockAcc.setDirectFlag(DirectFlagEnum.ADD.getKey());
            workerStockAcc.setChangeQty(bean.getQty());
            workerStockAcc.setCloseQty(bean.getQty());
            workerStockAcc.setRef("system");
            workerStockAcc.setObj("system");
            workerStockAcc.setCreateBy(adminUser.getNickName());
            workerStockAcc.setCreateTime(new Date());
            workerStockAcc.setFlag("INIT");
            workerStockAcc.setRemark("期初导入");
            workerStockAccs.add(workerStockAcc);

        }
        workerStockService.saveBatch(workerStocks);
        workerStockAccService.saveBatch(workerStockAccs);
    }

}
